

--[[
Metamorf Nameplates

'Morf' for short


-- Performance?
PlateHandler:SetFrameStrata("TOOLTIP")

-- Consider using UI Scale with the Hub?


The key differences with Metasequoia:
	- Each nameplate gets its own OnUpdate handler
	- Each nameplates gets its own queue
	- The main queue and onupdate handler is maintained, creating a hybrid system
	- Mouseovers will be handled by the main handler?
	- Simpler Update/Gather functions (UpdateUnitIdentity, UpdateUnitCondition, UpdateUnitContext, Etc.)
	- Optional Anchoring Tech

Nameplate Addon Parts

*1. Plate Detection
*2. Init Function to patch-in our textures and code
*3. Plate.OnUpdate handler to detect mouseover events, alpha, etc.
~5. Data Gathering Functions
6. Styler
7. Event Handlers for Target, Level, Threat, etc. Gamestate changes.
8. Indicator/Delegate Functions  (Indicator functions should be QUEUEABLE)
9. Widget Integration

* Should Widgets have their own carrier?

--
UpdateHealth
UpdateThreat
UpdateIdentity (name class etc)
UpdateSpellcast
UpdateContext
OnInit
OnHide
OnShow
--
Metasequoia Core
PLAYER_ENTERING_WORLD
PLAYER_TARGET_CHANGED
PLAYER_REGEN_ENABLED
PLAYER_REGEN_DISABLED
RAID_TARGET_UPDATE
UNIT_THREAT_SITUATION_UPDATE
UNIT_LEVEL

PLAYER_CONTROL_LOST
PLAYER_CONTROL_GAINED
UNIT_FACTION

UNIT_SPELLCAST_START, etc.

GameState table?

--]]


--[[
--------------------------------------------------------------------------------------------------------------
-- IX. External Commands: Allows widgets and themes to request updates to the plates.
-- Useful to make a theme respond to externally-captured data (such as the combat log)
--------------------------------------------------------------------------------------------------------------
function TidyPlates:ForceUpdate() SetMassQueue(Event_ResetNameplate) end
function TidyPlates:Update() SetMassQueue(Event_OnUpdateNameplate) end
function TidyPlates:RequestWidgetUpdate(plate) if plate then SetTargetQueue(plate, Event_RequestWidgetUpdate) else SetMassQueue(Event_RequestWidgetUpdate) end end
function TidyPlates:RequestDelegateUpdate(plate) if plate then SetTargetQueue(plate, Event_RequestDelegateUpdate) else SetMassQueue(Event_RequestDelegateUpdate) end end
function TidyPlates:ActivateTheme(theme) if theme and type(theme) == 'table' then TidyPlates.ActiveThemeTable, activetheme = theme, theme; SetMassQueue(Event_ResetNameplate) end end
function TidyPlates:EnableFadeIn() EnableFadeIn = true; end
function TidyPlates:DisableFadeIn() EnableFadeIn = nil; end
TidyPlates.StartCastAnimationOnNameplate = UpdateIndicator_StartCastAnimation
TidyPlates.StopCastAnimationOnNameplate = UpdateIndicator_StopCastAnimation
TidyPlates.NameplatesByGUID, TidyPlates.NameplatesAll, TidyPlates.NameplatesByVisible = GUID, Plates, PlatesVisible
--]]

TidyPlates = {}
local AddonName, TidyPlatesNamespace = ...
local TidyPlatesCore = CreateFrame("Frame")
local Nameplates = {}
local VisibleFrames = {}
local enableDirectMove, enableTransitions = false, true
local EMPTY_TEXTURE =  "Interface\\Addons\\TidyPlates\\Media\\Empty"

-----------------------------------------------------------------------------------------------
-- Declarations
-----------------------------------------------------------------------------------------------
-- New Plate Init Function Declarations
local InitFrame, InitVisible

-- Plate Frame Handler Declarations
local OnUpdateFrame, OnShowFrame, OnHideFrame
local OnShowPlate, OnHidePlate, OnChangePlate

-- Data Gather Declarations
local UpdateUnitIdentity, UpdateUnitCondition, UpdateUnitActivity, UpdateUnitContext

-- Queue Handling Functions
local QueueFunction, QueueFunctionAll

-- Indicators & Style Declarations
local UpdateFrameStyle, UpdateFrameIndicators

-----------------------------------------------------------------------------------------------
-- Plate Detection: Searches for new nameplates, and passes them to InitFrame()
-----------------------------------------------------------------------------------------------

do
	local WorldFrame = WorldFrame
	local lastUpdate, childCount = 0, 0
	
	local function IsNameplate(frame)		
		local bars = frame:GetChildren()
		if bars then 
			local border = select(2, bars:GetRegions() )
			return	border and
				border.GetTexture and
				border:GetTexture() == "Interface\\Tooltips\\Nameplate-Border"
		end
	end
	
	local function SearchForNewPlates(self, t)
		lastUpdate = lastUpdate + t

		if lastUpdate >= .03 then
			lastUpdate = 0
			local n = select('#', WorldFrame:GetChildren())

			if n ~= childCount then
				local i, f

				for i = 1, n do
					f = select(i, WorldFrame:GetChildren())
					if IsNameplate(f) and not f.TidyPlatesFrame then
						
						InitFrame(f)
					end
				end

				childCount = n
			end
		end
	end
	
	TidyPlatesCore:SetFrameStrata("TOOLTIP")
	TidyPlatesCore:SetScript("OnUpdate", SearchForNewPlates)
end

-----------------------------------------------------------------------------------------------
-- InitFrame: Hook Plate & Add Custom Frames, Passes off to the Show() handler
-----------------------------------------------------------------------------------------------

-- InitFrame: Called by detector function; Creates the complimentary frame; Hooks functions 
function InitFrame(plate)
	Nameplates[plate] = true
	
	-- Get References to the Blizzard Plate Cihldren and Regions
	local overlayChild, nameTextChild  = plate:GetChildren()
	local healthBar, castBar			 = overlayChild:GetChildren()
	
	local threatGlowRegion, overlayRegion,
		highlightRegion, levelTextRegion, 
		bossIconRegion, raidIconRegion,
		eliteIconRegion
		 = overlayChild:GetRegions()

	local _, castbarOverlay, shieldedRegion, spellIconRegion = castBar:GetRegions()	

	local nameTextRegion	= nameTextChild:GetRegions()
	
	local Blizzard = {
		["threatGlowRegion"] = threatGlowRegion, 
		["overlayRegion"] = overlayRegion,
		["highlightRegion"] = highlightRegion,
		["levelTextRegion"] = levelTextRegion, 
		["bossIconRegion"] = bossIconRegion, 
		["raidIconRegion"] = raidIconRegion,
		["eliteIconRegion"] = eliteIconRegion,
		["castbarOverlay"] = castbarOverlay, 
		["shieldedRegion"] = shieldedRegion,
		["spellIconRegion"] = spellIconRegion,
		["nameTextRegion"] = nameTextRegion,
		["healthBar"] = healthBar, 
		["castBar"] = castBar,	
		["overlayChild"] = overlayChild,
	}
	
	-- [[
	-- Make the Blizzard Nameplate Invisible
	threatGlowRegion:SetTexCoord( 0, 0, 0, 0 )        
	overlayRegion:SetTexCoord( 0, 0, 0, 0 )                     
	bossIconRegion:SetTexCoord( 0, 0, 0, 0 )         
	eliteIconRegion:SetTexCoord( 0, 0, 0, 0 )         
	nameTextRegion:SetWidth( .1 )                      
	levelTextRegion:SetWidth( .1 )                     
	spellIconRegion:SetTexCoord( 0, 0, 0, 0 )         
	raidIconRegion:SetAlpha( 0 )                      
	highlightRegion:SetTexture(nil)
	castbarOverlay:SetTexCoord( 0, 0, 0, 0 )
	shieldedRegion:SetTexCoord( 0, 0, 0, 0 )
	
	healthBar:SetStatusBarTexture(EMPTY_TEXTURE)
	castBar:SetStatusBarTexture(EMPTY_TEXTURE)
	--]]
	
	-- Create Tidy Plates Frame
	local frame = CreateFrame("Frame", nil, WorldFrame)
	frame:SetFrameStrata("BACKGROUND")
	frame.lastUpdate = 0
	frame.Plate = plate
	
	plate.TidyPlatesFrame = frame
	healthBar.TidyPlatesFrame = frame
	frame.ParentAnchor = overlayChild
	
	frame.Queue, frame.Unit, frame.UnitCache, frame.Visible = {}, {}, {}, {}
	frame.Blizzard, frame.Style  = 	Blizzard, nil
	
	frame:SetScript("OnShow", OnShowFrame)
	frame:SetScript("OnHide", OnHideFrame)
	frame:SetScript("OnUpdate", OnUpdateFrame)
	
	healthBar:HookScript("OnShow", OnShowPlate )
	healthBar:HookScript("OnHide", OnHidePlate )
	healthBar:HookScript("OnValueChanged", OnChangePlate ) 
	healthBar:HookScript("OnMinMaxChanged", OnChangePlate )
	
	-- Position
	if not enableDirectMove then frame:SetPoint("CENTER", overlayRegion, "CENTER") end
	frame:SetSize(100, 25)

	-- Alpha
	frame.requestedAlpha, frame.currentAlpha = 0, 0
	frame:SetAlpha(0)
	
	-- Scale
	frame.requestedScale, frame.currentScale = 1, 1
	
	InitVisible(frame)
	OnShowFrame(frame)
end

-- InitVisible:  Visible Textures & FontStrings
function InitVisible(frame)
	local visible = frame.Visible
	
	healthBar = CreateFrame("Frame", nil, frame)
	castBar = CreateFrame("Frame", nil, frame)
	
	-- Health Bar/Lowest Frame
	visible.healthborder = healthBar:CreateTexture(nil, "ARTWORK")
	visible.threatborder = healthBar:CreateTexture(nil, "ARTWORK")
	visible.highlight = healthBar:CreateTexture(nil, "OVERLAY")
	-- Extended/Tidy Plates Root Frame
	visible.raidicon = healthBar:CreateTexture(nil, "ARTWORK")
	visible.eliteicon = healthBar:CreateTexture(nil, "OVERLAY")
	visible.skullicon = healthBar:CreateTexture(nil, "OVERLAY")
	visible.target = healthBar:CreateTexture(nil, "BACKGROUND")       -- ARTWORK
	-- Cast Bar Frame
	visible.castborder = castBar:CreateTexture(nil, "ARTWORK")
	visible.castnostop = castBar:CreateTexture(nil, "ARTWORK")
	visible.spellicon = castBar:CreateTexture(nil, "OVERLAY")
	
	visible.raidicon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")	
	visible.highlight:SetAllPoints(visible.healthborder)
	visible.highlight:SetBlendMode("ADD")  

	local i, v; for i, v in pairs(visible) do v:SetNonBlocking(true) end

	-- Formatted Text
	-------------------------------------------------------------------------------------
	visible.customtext = frame:CreateFontString(nil, "OVERLAY")
	visible.name  = healthBar:CreateFontString(nil, "OVERLAY")
	visible.level = healthBar:CreateFontString(nil, "OVERLAY")
	-- Cast Bar Frame
	visible.spelltext = castBar:CreateFontString(nil, "OVERLAY")
end
	
-----------------------------------------------------------------------------------------------
-- Plate Frame Handlers
-----------------------------------------------------------------------------------------------

-- OnShowFrame:
function OnShowFrame(frame)
	local b = frame.Blizzard
	local v = frame.Visible
	
	UpdateUnitContext(frame, true)
	UpdateUnitIdentity(frame, true)
	UpdateUnitCondition(frame)
	
	VisibleFrames[frame] = true
end

-- OnHideFrame: 
function OnHideFrame(frame)
	frame.requestedAlpha = 0
	frame.currentAlpha = 0
	frame:SetAlpha(0)
	
	frame.requestedScale = 1
	frame.currentScale = 1
	frame:SetScale(1)
	
	VisibleFrames[frame] = nil
end

-- OnUpdate Handler: Visible nameplates call OnUpdateFrame() on every frame cycle.
do
	local currentTarget, currentMouseover = nil, nil
	local lastTarget, lastMouseover = nil, nil
	
	-- Helper Function Declarations
	local GetTransition
	
	-- OnUpdateFrame: Called frequently by each visible nameplate
	function OnUpdateFrame(frame, e)
		local unit = frame.Unit
		
		-- Mouseover
		if frame.Blizzard.highlightRegion:IsShown() then
			if not unit.isMouseover then
				unit.isMouseover = true
				QueueFunction(frame, Event_OnMouseover)
			end
		elseif unit.isMouseover then
			unit.isMouseover = false
			QueueFunction(frame, Event_OnMouseover)
		end
		
		-- Target
		if frame.Plate:GetAlpha() > .9 and UnitExists("target") then
			if not unit.isTarget then
				unit.isTarget = true
				QueueFunction(frame, GatherData_Alpha)
			end
		elseif unit.isTarget then
			unit.isTarget = false
			QueueFunction(frame, GatherData_Alpha)
		end
		
		-- Alpha Transition
		if  frame.currentAlpha ~= frame.requestedAlpha then
			frame.currentAlpha = GetTransition(frame.currentAlpha, frame.requestedAlpha, e, 4)
			frame:SetAlpha(frame.currentAlpha)
		end

		-- Scale Transition
		if frame.currentScale ~= frame.requestedScale then
			frame.currentScale =  GetTransition(frame.currentScale, frame.requestedScale, e, 4)
			frame:SetScale(frame.currentScale)
		end

		-- Position
		if enableDirectMove then
			local x = frame.ParentAnchor:GetLeft() + frame.ParentAnchor:GetWidth()/2
			local y = frame.ParentAnchor:GetBottom() + frame.ParentAnchor:GetHeight()/2
			frame:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', floor(x), floor(y))
		end
		
		-- Function Queue
		local queue, func = frame.Queue, nil
		
		for func in pairs(queue) do
			func(frame)
			queue[func] = nil
		end
	end	-- End of OnUpdateFrame

	-- GetTransition: 
	function GetTransition(current, requested, e, rate)
			if enableTransitions then 
				local delta = current - requested
				if delta > (e * rate) then
					return current - (e * rate)
				elseif delta < -(e * rate) then
					return current + (e * rate)
				end
			end
			return requested
	end

	-- QueueFunction:
	function QueueFunction(frame, func)
		if func and frame.Queue then frame.Queue[func] = true end
	end

	-- QueueFunctionAll:
	function QueueFunctionAll(func)
		if func then
			for f in pairs(VisibleFrames) do
				f.Queue[func] = true
			end
		end
	end

end

-- OnShowPlate, OnHidePlate:  Redirects events from the Blizzard Nameplate
function OnShowPlate(plate) plate.TidyPlatesFrame:Show() end
function OnHidePlate(plate) plate.TidyPlatesFrame:Hide() end

-- Health Changes
function OnChangePlate(plate) UpdateUnitCondition(plate.TidyPlatesFrame) end

-----------------------------------------------------------------------------------------------
-- Data Gather
-----------------------------------------------------------------------------------------------

do
	-- Helper function declaration
	local ColorToString, GetUnitReaction, GetUnitAggroStatus, GetUnitCombatStatus
	local ClassReference = {}
	
	-- UpdateUnitIdentity: Gathers Low-volatility information
	function UpdateUnitIdentity(frame, skipChanges)
		local unit = frame.Unit
		local bliz = frame.Blizzard
			
		-- Name
		unit.name = bliz.nameTextRegion:GetText()
		
		-- Skull
		unit.isBoss = bliz.bossIconRegion:IsShown()
		
		-- Level
		if unit.isBoss then 
			unit.level = "??"
			unit.levelcolorRed, unit.levelcolorGreen, unit.levelcolorBlue = 1, 0, 0
		else 
			unit.level = bliz.levelTextRegion:GetText() 
			unit.levelcolorRed, unit.levelcolorGreen, unit.levelcolorBlue = bliz.levelTextRegion:GetTextColor()
		end

		-- Elite
		unit.isElite = (bliz.eliteIconRegion:IsShown() or 0) == 1
		
		-- Reaction/Type
		unit.red, unit.green, unit.blue = bliz.healthBar:GetStatusBarColor()
		unit.reaction, unit.type = GetUnitReaction(unit.red, unit.green, unit.blue)
		
		-- Class
		unit.class = ClassReference[ColorToString(unit.red, unit.green, unit.blue)] or "UNKNOWN"
		
		
		
		-- Check for Changes, Sync Unit Cache
		-- Queue Updates
		-- local function ScanCache()
		-- QueueFunction( frame, UpdateFrameStyle)
		if skipChanges then return end
		UpdateFrameStyle(frame)
		UpdateFrameIndicators(frame)
		
		
		--QueueFunction( frame, UpdateFrameIndicators)
	end
	
	-- UpdateUnitCondition: Gathers information about the variable condition of a unit
	function UpdateUnitCondition(frame, skipChanges)
		local unit = frame.Unit
		local bliz = frame.Blizzard

		-- Health
		unit.health = bliz.healthBar:GetValue() or 0
		unit.healthmax = select(2, bliz.healthBar:GetMinMaxValues())
		
		-- Threat
		if InCombatLockdown() then
			unit.threatSituation, unit.threatValue = GetUnitAggroStatus(bliz.threatGlowRegion) 
		else unit.threatSituation = "LOW"; unit.threatValue = 0 end
		
		-- In Combat
		unit.isInCombat = GetUnitCombatStatus(bliz.nameTextRegion:GetTextColor())
		
		-- Raid Marked
		unit.isMarked = bliz.raidIconRegion:IsShown() or false
		if unit.isMarked then 
			ux, uy = bliz.raidIconRegion:GetTexCoord()
			unit.raidIconRegion = RaidIconCoordinate[ux][uy]
		else unit.raidIconRegion = nil end
		
		-- Check for Changes, Sync Unit Cache
		-- Queue Updates
		if skipChanges then return end
		UpdateFrameStyle(frame)
		UpdateFrameIndicators(frame)
		
		--QueueFunction( frame, UpdateFrameIndicators)
	end

	-- 
	function UpdateUnitActivity(frame, skipChanges)
		-- Casting
	end

	-- 
	function UpdateUnitContext(frame)
		local unit = frame.Unit
		local bliz = frame.Blizzard
		
		-- Update and Store GUID
		
		if skipChanges then return end

		--if unit.isTarget then
			-- Update Everything?
		--end
		
		-- ** 	Target and Mouseover data is gathered by the OnUpdate handler		**
		-- ** 	This function only needs to call the delegate. 					**
		
		-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		-- DELEGATE TEST
		if unit.isMouseover or unit.isTarget then
			frame.requestedAlpha = 1
		else
			frame.requestedAlpha = .5
		end

		if unit.isTarget then
			frame.requestedScale = 1.1
		else
			frame.requestedScale = .8
		end
	end
	
	-- ColorToString: Converts a color to a string with a C- prefix
	function ColorToString(r,g,b) return "C"..math.floor((100*r) + 0.5)..math.floor((100*g) + 0.5)..math.floor((100*b) + 0.5) end
	
	-- GetUnitReaction: Determines the reaction, and type of unit from the health bar color
	function GetUnitReaction(red, green, blue)
		if red < .01 then 	-- Friendly
			if blue < .01 and green > .99 then return "FRIENDLY", "NPC"
			elseif blue > .99 and green < .01 then return "FRIENDLY", "PLAYER"
			end
		elseif red > .99 then
			if blue < .01 and green > .99 then return "NEUTRAL", "NPC"
			elseif blue < .01 and green < .01 then return "HOSTILE", "NPC"
			end
		elseif red > .5 and red < .6 then
			if green > .5 and green < .6 and blue > .5 and blue < .6 then return "TAPPED", "NPC" end 	-- .533, .533, .99	-- Tapped Mob
		end
		return "HOSTILE", "PLAYER"
	end

	-- Populates the class color lookup table
	for classname, color in pairs(RAID_CLASS_COLORS) do 
		local r, g, b = color.r, color.g, color.b
		local colorstring = ColorToString(r, g, b)
		ClassReference[colorstring] = classname 
	end
	ClassReference["C010060"] = "MONK"
	
	-- Determines if a unit is in combat by checking the name text color
	function GetUnitCombatStatus(r, g, b) return (r > .5 and g < .5) end
	
	-- Determines if a unit is attacking, by looking at aggro glow region
	function GetUnitAggroStatus( region)
		-- High = 1, 0, 0	-- Medium High = 1, .6, 0  -- Medium Low = 1, 1, .47
		if not region:IsShown() then return "LOW", 0 end
		local red, green, blue = region:GetVertexColor()
		if red > 0 then
			if green > 0 then
				if blue > 0 then return "MEDIUM", 1 end
				return "MEDIUM", 2
			end
			return "HIGH", 3
		end
	end
	
end

-----------------------------------------------------------------------------------------------
-- Indicator Update
-----------------------------------------------------------------------------------------------

function UpdateFrameIndicators(frame)
	local v = frame.Visible
	local unit = frame.Unit
	
	-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	-- DELEGATE TEST
	v.name:SetText(unit.name.."@"..unit.health.."~"..unit.threatValue)
	v.name:SetTextColor(unit.red, unit.green, unit.blue)
end


-----------------------------------------------------------------------------------------------
-- Styler
-----------------------------------------------------------------------------------------------

do
	-- Helper Function Declaration
	local SetFontGroupObject, SetAnchorGroupObject, SetTextureGroupObject, SetBarGroupObject
	
	-- Style Property Groups
	local fontGroup = {"name", "level", "spelltext", "customtext"}
	local anchorGroup = {"healthborder", "threatborder", "castborder", "castnostop",
					"name",  "spelltext", "customtext", "level",
					"spellicon", "raidicon", "skullicon", "eliteicon", "target"}		
	local barGroup = {"castbar", "healthbar"}
	local textureGroup = { "castborder", "castnostop", "healthborder", "threatborder", "eliteicon", 
					"skullicon", "highlight", "target", "spellicon", }
							
				
	-- Styler function
	function UpdateFrameStyle(frame)

		local i
		local v = frame.Visible
		local s = frame.Style
		local unit = frame.Unit
		local objectstyle, objectname, objectregion, objectenable
		local plateAnchor = frame.Blizzard.overlayRegion
		
		-- Frame
		SetAnchorGroupObject(frame, s.frame, frame.plateAnchor)                      -- STAGE 1

		-- Anchorgroup
		for i, n in pairs(anchorGroup) do 
			SetAnchorGroupObject(v[n], s[n], v) 
			if s[n].show then v[n]:Show() else v[n]:Hide() end
		end
		-- Bars
		for i, n in pairs(barGroup) do SetBarGroupObject(bars[n], s[n], frame) end
		-- Texture
		for i, n in pairs(textureGroup) do SetTextureGroupObject(v[n], s[n]) end
		-- Font Group
		for i, n in pairs(fontGroup) do SetFontGroupObject(v[n], s[n]) end
		
		-- Hide Stuff
		if unit.isElite then v.eliteicon:Hide() else v.eliteicon:Hide() end
		if unit.isBoss then v.level:Hide() else v.skullicon:Hide() end
		if not unit.isTarget then v.target:Hide() end
		if not unit.isMarked then v.raidicon:Hide() end
		
		--[[ -- STYLER TEST
		v.name:SetFont(NAMEPLATE_FONT, 12, "NONE")
		v.name:SetPoint("CENTER", frame, "CENTER")
		v.name:SetSize(200, 32)
		--]]
	end
	
	-- Styler Helper Functions
	local function SetObjectShape(object, width, height) object:SetWidth(width); object:SetHeight(height) end
	local function SetObjectFont(object,  font, size, flags) if not object:SetFont(font, size, flags) then 
			object:SetFont("FONTS\\ARIALN.TTF", size or 12, flags) end end
	local function SetObjectJustify(object, horz, vert) object:SetJustifyH(horz); object:SetJustifyV(vert) end
	local function SetObjectAnchor(object, anchor, anchorTo, x, y) object:ClearAllPoints();object:SetPoint(anchor, anchorTo, anchor, x, y) end
	local function SetObjectTexture(object, texture) object:SetTexture(texture) end 
	local function SetObjectBartexture(obj, tex, ori, crop) obj:SetStatusBarTexture(tex); obj:SetOrientation(ori); end
	local function SetObjectShadow(object, shadow) if shadow then object:SetShadowColor(0,0,0, tonumber(shadow) or 1);
			object:SetShadowOffset(.5, -.5) else object:SetShadowColor(0,0,0,0) end; end
	
	-- SetFontGroupObject
	function SetFontGroupObject(object, objectstyle)
		if objectstyle then
			SetObjectFont(object, objectstyle.typeface or "FONTS\\ARIALN.TTF",  objectstyle.size or 12, objectstyle.flags or "NONE")
			SetObjectJustify(object, objectstyle.align or "CENTER", objectstyle.vertical or "BOTTOM")
			SetObjectShadow(object, objectstyle.shadow or 1)
		end
	end
	-- SetAnchorGroupObject
	function SetAnchorGroupObject(object, objectstyle, anchorTo)
		if objectstyle and anchorTo then
			SetObjectShape(object, objectstyle.width or 128, objectstyle.height or 16) --end
			SetObjectAnchor(object, objectstyle.anchor or "CENTER", anchorTo, objectstyle.x or 0, objectstyle.y or 0)
		end
	end
	-- SetTextureGroupObject
	function SetTextureGroupObject(object, objectstyle)
		if objectstyle then
			if objectstyle.texture then SetObjectTexture(object, objectstyle.texture or EMPTY_TEXTURE) end
			object:SetTexCoord(objectstyle.left or 0, objectstyle.right or 1, objectstyle.top or 0, objectstyle.bottom or 1)
		end
	end
	-- SetBarGroupObject
	function SetBarGroupObject(object, objectstyle, anchorTo)
		if objectstyle then
			SetAnchorGroupObject(object, objectstyle, anchorTo)
			SetObjectBartexture(object, objectstyle.texture or EMPTY_TEXTURE, objectstyle.orientation or "HORIZONTAL")
			if objectstyle.backdrop then
				object:SetBackdropTexture(objectstyle.backdrop)
			end
			object:SetTexCoord(objectstyle.left, objectstyle.right, objectstyle.top, objectstyle.bottom)
		end
	end
end
-----------------------------------------------------------------------------------------------
-- Event Handlers
-----------------------------------------------------------------------------------------------
local EventFunctions = {}

do		  
	function EventFunctions:PLAYER_REGEN_ENABLED()  
		QueueFunctionAll(UpdateUnitIdentity, true)
		QueueFunctionAll(UpdateUnitCondition)
	end
		
	function EventFunctions:PLAYER_REGEN_DISABLED() 
		QueueFunctionAll(UpdateUnitIdentity, true)
		QueueFunctionAll(UpdateUnitCondition)
	end
	
	function EventFunctions:RAID_TARGET_UPDATE() 			QueueFunctionAll(UpdateUnitCondition); end
	function EventFunctions:UNIT_THREAT_SITUATION_UPDATE() 	QueueFunctionAll(UpdateUnitCondition); end
	
	function EventFunctions:UNIT_LEVEL() 					QueueFunctionAll(UpdateUnitIdentity); end
	function EventFunctions:PLAYER_CONTROL_LOST() 			QueueFunctionAll(UpdateUnitIdentity); end
	
	EventFunctions.PLAYER_CONTROL_GAINED = EventFunctions.PLAYER_CONTROL_LOST
	EventFunctions.UNIT_FACTION = EventFunctions.PLAYER_CONTROL_LOST	
end
	
do	
	local function OnEvent(self, event, ...) if event and EventFunctions[event] then EventFunctions[event](...) end; end
	TidyPlatesCore:SetScript("OnEvent", OnEvent)
	for event in pairs(EventFunctions) do TidyPlatesCore:RegisterEvent(event) end
end


